@@ -3,7 +3,7 @@
  * function:    Object set for handling vector images.                        *
  * modul:       vec_image.cc                                                  *
  * licency:     GPL or LGPL                                                   *
- * Copyright: (c) 2018-2021 Jaroslav Fojtik                                   *
+ * Copyright: (c) 2018-2022 Jaroslav Fojtik                                   *
  ******************************************************************************/
 #include <stdlib.h>
 #include <stdio.h>
@@ -70,7 +70,10 @@ PS_State::PS_State(void)
 {
   PS_StateC *pPS_StateC = (PS_StateC *)this;
   memset(pPS_StateC,0,sizeof(PS_StateC));
+  FillTransparency = 0xFF;
+  HatchDistance = 1;
   memset(&PaperBackground, 0, sizeof(PaperBackground));
+  memset(&FillBackground, 0xFF, sizeof(FillBackground)); //Use white paper.
   pPalette = NULL;
 }
 
@@ -115,6 +118,8 @@ void PS_State::AttachPalette(APalette *NewPalette)
 vecPen::vecPen(void)
 {
   LineStyle = 1;
+  LineCap = 0;
+  LineJoin = 0;
   PenWidth = -1;
 }
 
@@ -133,6 +138,16 @@ void vecPen::prepExport(PS_State *PSS) const
       PSS->LineStyle = LineStyle;
       PSS->dirty |= PSS_LineStyle;
     }
+    if(PSS->LineCap != LineCap)
+    {
+      PSS->LineCap = LineCap;
+      PSS->dirty |= PSS_LineCap;
+    }
+    if(PSS->LineJoin != LineJoin)
+    {
+      PSS->LineJoin = LineJoin;
+      PSS->dirty |= PSS_LineJoin;
+    }
     if(PenWidth>=0 && PSS->LineWidth!=PenWidth)
     {      
       PSS->LineWidth = PenWidth;
@@ -146,6 +161,8 @@ void vecPen::AttribFromPSS(const PS_State &PSS)
 {
   memcpy(&LineColor, &PSS.LineColor, sizeof(PSS.LineColor));
   LineStyle = PSS.LineStyle;
+  LineCap = PSS.LineCap;
+  LineJoin = PSS.LineJoin;
   PenWidth = PSS.LineWidth;
 }
 
@@ -154,6 +171,7 @@ vecBrush::vecBrush(const PS_State &PSS)
 {
   BrushStyle = PSS.FillPattern;
   memcpy(&FillColor, &PSS.FillColor, sizeof(PSS.FillColor));
+  memcpy(&FillBackground, &PSS.FillBackground, sizeof(FillBackground));
 }
 
 
@@ -168,6 +186,10 @@ void vecBrush::prepExport(PS_State *PSS) const
     }
     if(BrushStyle != PSS->FillPattern)
         PSS->FillPattern = BrushStyle;
+    //if(memcmp(&FillBackground, &PSS->FillBackground, sizeof(FillBackground))!=0)
+    //{  // No need to compare.
+      memcpy(&PSS->FillBackground, &FillBackground, sizeof(FillBackground));
+    //}
   }
 }
 
@@ -175,18 +197,22 @@ void vecBrush::prepExport(PS_State *PSS) const
 void vecBrush::AttribFromPSS(const PS_State &PSS)
 {
   memcpy(&FillColor, &PSS.FillColor, sizeof(FillColor));
+  memcpy(&FillBackground, &PSS.FillBackground, sizeof(FillBackground));
   BrushStyle = PSS.FillPattern;  
 }
 
 
 void vecBrush::FeaturesEPS(unsigned & Feature) const 
 {
-  if(BrushStyle==106) Feature |= EPS_FillBox1;
-  if(BrushStyle==107) Feature |= EPS_FillBox2;
-  if(BrushStyle==108) Feature |= EPS_FillPlus;
-  if(BrushStyle==109) Feature |= EPS_FillBalls;
-  if(BrushStyle==111) Feature |= EPS_FillTriangle;
-  if(BrushStyle==112) Feature |= EPS_FillSmSquare;
+  switch(BrushStyle & 0x7F)
+  {
+    case FILL_SQUARES:	Feature|=EPS_FillBox1; break;
+    case FILL_SQUARES2: Feature|=EPS_FillBox2; break;
+    case FILL_PLUS:	Feature|=EPS_FillPlus; break;
+    case FILL_BALLS:	Feature|=EPS_FillBalls; break;
+    case FILL_TRIANGLES:Feature|=EPS_FillTriangle; break;
+    case FILL_SM_SQUARES:Feature|=EPS_FillSmSquare; break;
+  }
 }
 
 
@@ -312,6 +338,16 @@ bool ContextSaved = false;
 
 ////////////////////////////////////
 
+
+void VectorObject::Export2EPS(FILE *F, PS_State *PSS) const
+{
+  temp_string StrTmp(Export2EPS(PSS));
+  if(StrTmp.length() > 0)
+      fputs(StrTmp(), F);
+}
+
+
+
 VectorEllipse::VectorEllipse(float iniBottomRect, float iniTopRect, float iniRightRect, float iniLeftRect)
 {
  Tx = NULL;
@@ -352,7 +388,7 @@ string str;
 			 fabs(RightRect-LeftRect)/2, fabs(TopRect-BottomRect)/2,
 			 bAngle, eAngle);
 
-  if(BrushStyle!=0)
+  if(BrushStyle!=FILL_NONE)
   {
     FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);
   }
@@ -413,7 +449,7 @@ string str;
 			 bAngle, eAngle);
   //str.cat_printf("\n%2.2f %2.2f lineto", (RightRect+LeftRect)/2, (TopRect+BottomRect)/2);
 
-  if(BrushStyle!=0)
+  if(BrushStyle!=FILL_NONE)
   {
     FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);
   }
@@ -451,7 +487,7 @@ VectorRectangle::~VectorRectangle()
 
 unsigned VectorRectangle::isInside(float xp, float yp) const
 {
-  if(BrushStyle==0) return 0;
+  if(BrushStyle==FILL_NONE) return 0;
   if(xp>=LeftRect && xp<=RightRect && yp>=BottomRect && xp<=TopRect)
   {
     if(xp>LeftRect && xp<RightRect && yp>BottomRect && xp<TopRect) return 2;
@@ -487,7 +523,7 @@ float x, y;
   str.cat_printf("\n%2.2f %2.2f lineto", x, y);
   str.cat_printf("\nclosepath");
 
-  if(BrushStyle!=0)
+  if(BrushStyle!=FILL_NONE)
   {
     FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);    
   }
@@ -739,7 +775,7 @@ unsigned Intersects = 0;
 bool TouchLine = false;
 int OriUpDn = 0;
 
-  if(CountPoints<=2 || Points==NULL || !Close || BrushStyle==0) return 0;
+  if(CountPoints<=2 || Points==NULL || !Close || BrushStyle==FILL_NONE) return 0;
 
   const float *pEnd = Points + 2*(CountPoints-1);
   for(float *p=Points; p<pEnd; p+=2)
@@ -824,7 +860,7 @@ unsigned Intersects = 0;
 bool TouchLine = false;
 int OriUpDn = 0;
 
-  if(CountPoints<=2 || Points==NULL || !Close || BrushStyle==0) return 0;
+  if(CountPoints<=2 || Points==NULL || !Close || BrushStyle==FILL_NONE) return 0;
 
   const float *pEnd = Points + 2*(CountPoints-1);
   for(float *p=Points; p<pEnd; p+=2)
@@ -976,7 +1012,7 @@ float *p = Points;
     }
   }
 
-  if(Filled && BrushStyle!=0)
+  if(Filled && BrushStyle!=FILL_NONE)
   {
     str += "\nclosepath";
     FillObjectPS(MinPolyX,MaxPolyX,MinPolyY,MaxPolyY,str,PSS);
@@ -1210,9 +1246,7 @@ VectorObject *IterObject = FirstObject;
   if(F==NULL) return;
   while(IterObject!=NULL)
   {
-    string StrTmp(IterObject->Export2EPS(PSS));
-    if(StrTmp.length() > 0)
-      fputs(StrTmp(), F);
+    IterObject->Export2EPS(F,PSS);
     IterObject = IterObject->NextObject;
   }
 }
@@ -1455,6 +1489,82 @@ return temp_string(str);
 }
 
 
+void TextContainer::CalculateExtent(float &TextWidth, float &TextHeight) const
+{
+float CurPosX = 0;
+float LastLineSz = 0;
+
+  TextWidth = TextHeight = 0;
+  if(Text!=NULL && TextObjects>0)
+  {
+    for(int i=0; i<TextObjects; i++)
+    {
+      if(Text[i]!=NULL)
+      {
+	LastLineSz = Text[i]->size;
+	int j = 0;
+
+	string str;
+	while(j < Text[i]->contents.length())
+        {
+	  char ch = Text[i]->contents[j];
+	  if(ch=='\\')
+	  {
+	    j++;
+            while(j < Text[i]->contents.length())
+	    {
+	      ch = Text[i]->contents[j];
+	      j++;
+	      if(ch==' ') break;
+	      if(ch=='{')
+	      {
+	        while(j < Text[i]->contents.length())
+		{
+		  ch = Text[i]->contents[j];
+	          j++;
+		  if(ch=='}') break;
+		  str += ch;
+		}
+	        break;
+	      }
+	    }
+	    continue;
+	  }
+	  str += ch;
+	  j++;
+        }
+
+	j = 0;
+	while(j < str.length())
+	{
+          char ch = str[j];
+          switch(ch)
+	  {
+	    case '\n': if(CurPosX>TextWidth) TextWidth=CurPosX;
+	               CurPosX = 0;
+	               TextHeight += LastLineSz;
+		       break;
+	    case 'l':
+	    case 'i':  CurPosX += 0.26 * LastLineSz;
+		       break;
+	    case 'm':  CurPosX += 0.71 * LastLineSz;
+		       break;
+            default:   if(isupper(ch))
+		         CurPosX += 0.5 * LastLineSz;
+		       else
+                         CurPosX += 0.44 * LastLineSz;
+		       break;
+	  }
+	  j++;
+	}
+      }
+    }
+    if(CurPosX>TextWidth) TextWidth=CurPosX;
+    TextHeight += LastLineSz;
+  }
+}
+
+
 #ifdef _DEBUG
 void TextContainer::Dump2File(FILE *F) const
 {
@@ -1749,6 +1859,16 @@ void PS_Attr(string & PSData, PS_State *PSS)
    PSData+=" setdash";
    PSS->dirty &= ~PSS_LineStyle;
    }
+ if(PSS->dirty & PSS_LineCap)
+  {
+  PSData.cat_printf("\n%u setlinecap",PSS->LineCap);
+  PSS->dirty &= ~PSS_LineCap;
+  }
+ if(PSS->dirty & PSS_LineJoin)
+  {
+  PSData.cat_printf("\n%u setlinejoin",PSS->LineJoin);
+  PSS->dirty &= ~PSS_LineJoin;
+  }
  if(PSS->dirty & PSS_LineWidth)
   {
   PSData.cat_printf("\n%2.2f setlinewidth",PSS->LineWidth);
@@ -1779,14 +1899,27 @@ float Distance;
     PSS->FirstTimeFix = false;
     }
 
+  if(PSS->FillPattern > (0x80|FILL_SOLID))
+  {
+    color.printf("\n%0.2g %0.2g %0.2g setrgbcolor",
+		(float)PSS->FillBackground.Red/256.0,
+	 	(float)PSS->FillBackground.Green/256.0,
+		(float)PSS->FillBackground.Blue/256.0);
+    PSData += "\ngsave";
+    PSData += color;
+    PSData += "\nfill";
+    PSData+=" grestore";
+    erase(color);
+  }
+
   color.printf("\n%0.2g %0.2g %0.2g setrgbcolor",
 		(float)PSS->FillColor.Red/256.0,
 	 	(float)PSS->FillColor.Green/256.0,
 		(float)PSS->FillColor.Blue/256.0);
 
-  switch(PSS->FillPattern)
-		{
-		case 1:if(PSS->LineStyle!=0)
+  switch(PSS->FillPattern & 0x7F)
+  {
+    case FILL_SOLID:if(PSS->LineStyle!=0)
 			 PSData+="\ngsave";
 		       PSData+=color;
 		       PSData+="\nfill";
@@ -1795,20 +1928,26 @@ float Distance;
 		       else
 			 PSS->dirty |= PSS_LineColor;
 		       break;
-		case 2:Distance = 2.8;
+
+    case FILL_DIAG_UPx:Distance = PSS->HatchDistance;
 		       goto FINALDiagonal;
-		case 3:Distance = 5.7;
+    case FILL_DIAG_UP: Distance = 2.8;
 		       goto FINALDiagonal;
-		case 4:Distance = 11.3;
+    case FILL_DIAG_UP2:Distance = 5.7;
+		       goto FINALDiagonal;
+    case FILL_DIAG_UP4:Distance = 11.3;
 FINALDiagonal:	       tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rlineto}",  // Diagonal up
 			       MinPolyX-(MaxPolyY-MinPolyY),Distance,MaxPolyX,MinPolyY,
 			       MaxPolyY-MinPolyY, MaxPolyY-MinPolyY  );
 		       goto FINALLY;
-		case 5:Distance=2.8;
+
+    case FILL_DIAG_CROSSHATCHx:Distance = PSS->HatchDistance;
 		       goto FINALCross;
-		case 6:Distance=5.7;
+    case FILL_DIAG_CROSSHATCH:Distance=2.8;
 		       goto FINALCross;
-		case 7:Distance=11.3;
+    case FILL_DIAG_CROSSHATCH2:Distance=5.7;
+		       goto FINALCross;
+    case FILL_DIAG_CROSSHATCH4:Distance=11.3;
 FINALCross:	       tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rlineto}for",
 			       MinPolyX-(MaxPolyY-MinPolyY),Distance,MaxPolyX,MinPolyY,
 			       MaxPolyY-MinPolyY, MaxPolyY-MinPolyY );
@@ -1821,11 +1960,14 @@ FINALCross:	       tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rli
 			       MinPolyX,Distance,(MaxPolyX+(MaxPolyY-MinPolyY)),MinPolyY,
 			       -(MaxPolyY-MinPolyY), MaxPolyY-MinPolyY  );
 			goto FINALLY2;
-		case 8: Distance=2;
+
+    case FILL_VERTICALx:Distance = PSS->HatchDistance;
 			goto FINALVertical;
-		case 9: Distance=4;
+    case FILL_VERTICAL: Distance=2;
 			goto FINALVertical;
-		case 10:Distance=8;
+    case FILL_VERTICAL2: Distance=4;
+			goto FINALVertical;
+    case FILL_VERTICAL4:Distance=8;
 FINALVertical:		tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto 0 %2.2f rlineto}",
 			       MinPolyX,Distance,MaxPolyX,MinPolyY,
 			       (MaxPolyY-MinPolyY));
@@ -1837,7 +1979,8 @@ FINALLY2:		PSData += tmp;
 			PSData += "for\nstroke";
 			if(PSS->LineStyle!=0) PSData+=" grestore";
 			break;
-		case 11:Distance=16;
+
+    case FILL_GRAY_3:	Distance=16;
 FINALDot:		tmp.printf("\n[0.5 %2.1f] 0 setdash\n %2.2f %2.1f %2.2f {%2.2f moveto 0 %2.2f rlineto %2.2f %2.2f rmoveto 0 %2.2f rlineto}",
 			       Distance-0.5,
 			       MinPolyX,Distance,MaxPolyX,
@@ -1846,32 +1989,44 @@ FINALDot:		tmp.printf("\n[0.5 %2.1f] 0 setdash\n %2.2f %2.1f %2.2f {%2.2f moveto
 			       Distance/2.0,-Distance/2.0 - (MaxPolyY-MinPolyY),
 			       (MaxPolyY-MinPolyY)+Distance/2.0  );
 			goto FINALLY;
-		case 12:Distance=8;
+
+    case FILL_GRAY_6: Distance=8;
 			goto FINALDot;
-		case 13:Distance=4;
+    case FILL_GRAY_12:Distance=4;
 			goto FINALDot;
-		case 14:Distance=2;
+    case FILL_GRAY_25:Distance=2;
 			goto FINALDot;
-		case 15:Distance=1;
+    case FILL_GRAY_50:Distance=1;
+			goto FINALDot;
+    case FILL_GRAY_75:Distance=0.75;
 			goto FINALDot;
 
-		case 100:Distance=2.8;
+    case FILL_DIAG_DOWN:Distance=2.8;
 FinalDiagDown:		tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rlineto}",
 			       MinPolyX,Distance,(MaxPolyX+(MaxPolyY-MinPolyY)), MinPolyY,
 			       -(MaxPolyY-MinPolyY), MaxPolyY-MinPolyY);
 			goto FINALLY;
-		case 101:Distance = 5.7;
+    case FILL_DIAG_DOWN2:Distance = 5.7;
+			goto FinalDiagDown;
+    case FILL_DIAG_DOWN4:Distance = 11.4;
+			goto FinalDiagDown;
+    case FILL_DIAG_DOWNx:Distance=PSS->HatchDistance;
 			goto FinalDiagDown;
 
-		case 102:Distance=2;
+    case FILL_HORIZONTAL:Distance=2;
 FINALHorizontal:	 tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f exch moveto %2.2f 0 rlineto}",
 			       MinPolyY,Distance,MaxPolyY, MinPolyX,
 			       (MaxPolyX-MinPolyX));
 			goto FINALLY;
-		case 103:Distance=4;
+    case FILL_HORIZONTAL2:Distance=4;
 			goto FINALHorizontal;
+    case FILL_HORIZONTAL4:Distance=8;
+			goto FINALHorizontal;
+    case FILL_HORIZONTALx:Distance=PSS->HatchDistance;
+			goto FINALHorizontal;
+			
 
-		case 104:Distance=2;
+    case FILL_CROSS_HATCH:Distance=2;
 FINALHrzCross:		 tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f exch moveto %2.2f 0 rlineto}for",
 			       MinPolyY,Distance,MaxPolyY, MinPolyX,
 			       (MaxPolyX-MinPolyX));
@@ -1884,10 +2039,14 @@ FINALHrzCross:		 tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f exch moveto %2.2f 0 rlin
 			       MinPolyX,Distance,MaxPolyX, MinPolyY,
 			       (MaxPolyY-MinPolyY));
 			goto FINALLY2;
-		case 105:Distance=4;
+    case FILL_CROSS_HATCH2:Distance=4;
+			goto FINALHrzCross;
+    case FILL_CROSS_HATCH4:Distance=8;
+			goto FINALHrzCross;
+    case FILL_CROSS_HATCHx:Distance=PSS->HatchDistance;
 			goto FINALHrzCross;
 
-		case 106:tmp = "BoxFill1";
+    case FILL_SQUARES:  tmp = "BoxFill1";
 FillPattern:		PSData+="\ngsave clip\n"
 				 "[/Pattern /DeviceRGB] setcolorspace";
 			PSData += copy(color,0,color.length()-11);
@@ -1896,43 +2055,44 @@ FillPattern:		PSData+="\ngsave clip\n"
 				"grestore";
 			if(PSS->LineStyle==0) PSData+="\nstroke";
 			break;
-		case 107:tmp = "BoxFill2";
+    case FILL_SQUARES2: tmp = "BoxFill2";
 			goto FillPattern;
-		case 108:tmp = "PlusFill";
+    case FILL_PLUS:	tmp = "PlusFill";
 			goto FillPattern;
-		case 109:tmp = "BallsFill";
+    case FILL_BALLS:	tmp = "BallsFill";
 			goto FillPattern;
-		case 110:Distance=10;
-			  PSData += '\n';
-			  if(PSS->LineStyle!=0) PSData+="gsave";
-			  PSData+=" clip newpath 0 setlinewidth\n";
-			  PSData+=color;
-			  tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f exch moveto %2.2f 0 rlineto",
+    case FILL_TARTAN:	Distance=10;
+			PSData += '\n';
+			if(PSS->LineStyle!=0) PSData+="gsave";
+			PSData+=" clip newpath 0 setlinewidth\n";
+			PSData+=color;
+			tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f exch moveto %2.2f 0 rlineto",
 			       MinPolyY,Distance,MaxPolyY, MinPolyX,
 			       (MaxPolyX-MinPolyX));
-			  PSData += tmp;
-			  tmp.printf(" 0 %2.2f rmoveto %2.2f 0 rlineto",
+			PSData += tmp;
+			tmp.printf(" 0 %2.2f rmoveto %2.2f 0 rlineto",
 			       Distance/6, -(MaxPolyX-MinPolyX));
-			  PSData += tmp;
-			  tmp.printf(" 0 %2.2f rmoveto %2.2f 0 rlineto}for",
+			PSData += tmp;
+			tmp.printf(" 0 %2.2f rmoveto %2.2f 0 rlineto}for",
 			       Distance/6, (MaxPolyX-MinPolyX));
-			  PSData += tmp;
+			PSData += tmp;
 
-			  tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto 0 %2.2f rlineto",
+			tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto 0 %2.2f rlineto",
 			       MinPolyX,Distance,MaxPolyX, MinPolyY,
 			       (MaxPolyY-MinPolyY));
-			  PSData += tmp;
-			  tmp.printf(" %2.2f 0 rmoveto 0 %2.2f rlineto",
+			PSData += tmp;
+			tmp.printf(" %2.2f 0 rmoveto 0 %2.2f rlineto",
 			       Distance/6, -(MaxPolyY-MinPolyY));
-		          PSData += tmp;
-			  tmp.printf(" %2.2f 0 rmoveto 0 %2.2f rlineto}",
+		        PSData += tmp;
+			tmp.printf(" %2.2f 0 rmoveto 0 %2.2f rlineto}",
 			       Distance/6, (MaxPolyY-MinPolyY));
-			  goto FINALLY2;
-		case 111:tmp = "TriangleFill";
+			goto FINALLY2;
+
+    case FILL_TRIANGLES:tmp = "TriangleFill";
 			 goto FillPattern;
-		case 112:tmp = "SmSquareFill";
+    case FILL_SM_SQUARES:tmp = "SmSquareFill";
 			 goto FillPattern;
-		}
+  }
 
   if(PSS->LineStyle!=0)
     PSData+="\nstroke";		//draw frame line
