@@ -3,7 +3,7 @@
  * Purpose: Prototypes for general manipulation vith vector images         *
  * Modul:   vecimage.h                                                     *
  * Licency: GPL or LGPL                                                    *
- * Copyright: (c) 1998-2021 Jaroslav Fojtik                                *
+ * Copyright: (c) 1998-2022 Jaroslav Fojtik                                *
  ***************************************************************************/
 #ifndef _VECIMAGE_H_
 #define _VECIMAGE_H_
@@ -12,10 +12,12 @@
 #include "raster.h"
 
 
-#define PSS_LineColor 1
-#define PSS_LineStyle 2
-#define PSS_LineWidth 4
-#define PSS_FillColor 8
+#define PSS_LineColor	 1
+#define PSS_LineStyle	 2
+#define PSS_LineWidth	 4
+#define PSS_FillColor	 8
+#define PSS_LineCap	16
+#define PSS_LineJoin	32
 
 #define mm2PSu(x)   ( (float)(x)*71/25.4 )
 
@@ -27,28 +29,88 @@ void Color2PS(string & Text, const RGB_Record & TextColor);
 class CpTranslator;
 
 
+typedef enum
+{
+  FILL_NONE = 0,		// Hollow
+  FILL_SOLID = 1,		// Solid
+  FILL_DIAG_UP = 2,		// Finely spaced 45-degree lines
+  FILL_DIAG_UP2 = 3,		// Medium spaced 45-degree lines
+  FILL_DIAG_UP4 = 4,            // Coarsely spaced 45-degree lines
+  FILL_DIAG_CROSSHATCH = 5,	// Fine 45-degree hatching
+  FILL_DIAG_CROSSHATCH2 = 6,	// Medium 45-degree hatching
+  FILL_DIAG_CROSSHATCH4 = 7,	// Coarse 45-degree hatching
+  FILL_VERTICAL = 8,		// Fine vertical lines
+  FILL_VERTICAL2 = 9,		// Medium vertical lines
+  FILL_VERTICAL4 = 10,		// Coarse vertical lines
+  FILL_GRAY_3 = 11,		// Dots density 1 (least dense)
+  FILL_GRAY_6 = 12,		// Dots density 2
+  FILL_GRAY_12 = 13,		// Dots density 3
+  FILL_GRAY_25 = 14,		// Dots density 4
+  FILL_GRAY_50 = 15,		// Dots density 5
+  FILL_GRAY_75 = 16,		// Dots density 6 (densest)
+				// 18 Dots (medium)
+				// 19 Dots (coarse)
+  FILL_HORIZONTAL = 20,		// Fine horizontal
+  FILL_HORIZONTAL2 = 21,	// Medium horizontal
+  FILL_HORIZONTAL4 = 22,	// Coarse horizontal
+  FILL_CROSS_HATCH = 23,	// Fine 90-degree cross-hatching
+  FILL_CROSS_HATCH2 = 24,	// Medium 90-degree cross-hatching
+  FILL_CROSS_HATCH4 = 25,	// Coarse 90-degree cross-hatching
+  FILL_DIAG_DOWN = 26,		// Fine 45-degree lines
+  FILL_DIAG_DOWN2 = 27,		// Medium 45-degree lines
+  FILL_DIAG_DOWN4 = 28,		// Coarse 45-degree lines
+				// 29 Brick pattern (horizontal)
+				// 30 Brick pattern (vertical)
+				// 31 NA
+				// 32 Interweaving
+				// 33 NA
+				// 34 NA
+				// 35 Tile pattern
+				// 36 Coarse lines (thick)
+  FILL_SQUARES = 37,		// Alternating dark and light squares
+
+  FILL_CROSS_HATCHx = 94,
+  FILL_DIAG_DOWNx = 95,
+  FILL_DIAG_CROSSHATCHx = 96,
+  FILL_DIAG_UPx = 97,
+  FILL_VERTICALx = 98,
+  FILL_HORIZONTALx = 99,
+  FILL_SQUARES2 = 107,
+  FILL_PLUS = 108,
+  FILL_BALLS = 109,
+  FILL_TARTAN = 110,
+  FILL_TRIANGLES = 111,
+  FILL_SM_SQUARES = 112
+} E_FILL_STYLES;
+
+
+/** C type PS state, it could be copied using memcpy. */
 typedef struct
 {
+	const CpTranslator *ConvertCpg;
 	float FontSize, FontSizeW;	///< Size of font in [mm].
+	float LineWidth;
+	float HatchDistance;		///< PSu
+
 	WORD FontWeight;
-	SWORD FontOrientation10;	///< Text rotation request.
-        const CpTranslator *ConvertCpg;
+	SWORD FontOrientation10;	///< Text rotation request.        
         
 	char FontItallic;
 	char dirty;
-	char FillPattern;
+	BYTE FillPattern;
+        BYTE FillTransparency;
 	BYTE LineStyle;
+	BYTE LineCap;			///< 0-butt caps, 1-round caps, 2-extended butt caps.
+	BYTE LineJoin;			///< 0-miter join, 1-round join, 2-bevel join.
         BYTE PolyFillMode;		///< 0-standard; 1-alternating
 	BYTE TextAllign;		///< 0-TA_LEFT; 2-TA_RIGHT; 6-TA_CENTER
 
-	float LineWidth;
-	RGB_Record FillColor;
-        BYTE FillTransparency;
+	RGB_Record FillColor;	
         RGB_Record FillBackground;
 
 	RGB_Record LineColor;
 	RGB_Record TextColor;
-
+	
         bool FirstTimeFix;
 } PS_StateC;
 
@@ -93,11 +155,21 @@ public:
 
 ///////////////////////////////////////////////////////////
 
+typedef enum
+{
+  ATTR_NONE =	0,
+  ATTR_PEN,
+  ATTR_BRUSH,
+  ATTR_PALETTE,
+} ATTRIB_TYPES;
+
+
 class VectorAttribute
 {
 public:
   virtual ~VectorAttribute() {}
   virtual void prepExport(PS_State *PSS) const = 0;
+  virtual int getAttribType(void) const {return ATTR_NONE;};
 };
 
 class vecPen: public VectorAttribute
@@ -105,12 +177,15 @@ class vecPen: public VectorAttribute
 public:
   RGB_Record LineColor;
   unsigned char LineStyle;
+  unsigned char LineCap;
+  unsigned char LineJoin;
   float PenWidth;
 
   vecPen(void);
   vecPen(const PS_State &PSS) {AttribFromPSS(PSS);}
 
   virtual void prepExport(PS_State *PSS) const;
+  virtual int getAttribType(void) const {return ATTR_PEN;};
   void AttribFromPSS(const PS_State &PSS);
 };
 
@@ -119,6 +194,7 @@ class vecBrush: public VectorAttribute
 {
 public:
   RGB_Record FillColor;
+  RGB_Record FillBackground;
   unsigned char BrushStyle;
 
   vecBrush() {BrushStyle=1;}
@@ -126,6 +202,7 @@ public:
 
   virtual void prepExport(PS_State *PSS) const;
   virtual void FeaturesEPS(unsigned & Feature) const;
+  virtual int getAttribType(void) const {return ATTR_BRUSH;};
   void AttribFromPSS(const PS_State &PSS);
 };
 
@@ -161,6 +238,7 @@ public:
   attrPalette(APalette *iniPalette);
   virtual ~attrPalette();
   virtual void prepExport(PS_State *PSS) const;
+  virtual int getAttribType(void) const {return ATTR_PALETTE;};
 };
 
 
@@ -187,6 +265,7 @@ class VectorObject
 {
 public:
   virtual temp_string Export2EPS(PS_State *PSS=NULL) const {return temp_string();}
+  virtual void Export2EPS(FILE *F, PS_State *PSS) const;
   virtual void Transform(const AbstractTransformXY &Tx) {}
   virtual void FeaturesEPS(unsigned & Feature) const {return;}
   virtual unsigned isInside(float xp, float yp) const {return 0;}
@@ -390,6 +469,7 @@ public:
   void AddText(temp_string contents, const PS_State &PSS);
   void AddText(temp_string contents, const char *font, const PS_State &PSS);
 
+  void CalculateExtent(float &TextWidth, float &TextHeight) const;
   bool isEmpty(void) const;
 
   TextObject **Text;
@@ -408,7 +488,45 @@ public:
   VectorImage(void);
   VectorImage(VectorList &VecList, const PS_State & NewPSS);
 
-  void Export2EPS(FILE *F) {VectorList::Export2EPS(F,&PSS);}
+  void Export2EPS(FILE *F) {PSS.LineJoin=PSS.LineCap=0;VectorList::Export2EPS(F,&PSS);}
+};
+
+
+//////////////////////////////////////////////////////
+
+void DumpRaster2File(FILE *f, Raster2DAbstract *Raster, APalette *Palette,
+		float CenterX, float CenterY,
+		float dx, float dy, float RotAngle, int ColorMode);
+
+
+class VectorRaster: public VectorObject
+{
+public:
+  VectorRaster(float iniBottomRect, float iniTopRect, float iniRightRect, float iniLeftRect);
+  VectorRaster(float iniCenterX, float iniCenterY, float dx, float dy, float alpha);
+  VectorRaster(Raster2DAbstract *iniRaster, float iniCenterX, float iniCenterY, float TrDx, float TrDy, float alpha);
+  virtual ~VectorRaster();
+
+  virtual temp_string Export2EPS(PS_State *PSS=NULL) const;  
+  virtual void Export2EPS(FILE *F, PS_State *PSS=NULL) const;
+  virtual void Transform(const AbstractTransformXY &Tx);
+  virtual void FeaturesEPS(unsigned & Feature) const;
+
+  void AttachRaster(Raster2DAbstract *NewRaster);
+  void AttachPalette(APalette *NewPalette);
+  IMAGE_TYPE ImageType(void) const;
+
+  float CenterX, CenterY;
+  float RightCenterX, RightCenterY;
+  float TopCenterX, TopCenterY;
+
+#ifdef _DEBUG
+  void Dump2File(FILE *F) const;
+#endif
+
+protected:
+  Raster2DAbstract *Raster;
+  APalette *Palette;
 };
 
 
