@@ -108,13 +108,13 @@ void TconvertedPass1_602::HeaderFooter602(const string &args, char He_Fo)
   recursion++;
 
   line_term = 's';		//Soft return
-  if(char_on_line == -20)   /* Left one empty line for new enviroment */
+  if(char_on_line == LEAVE_ONE_EMPTY_LINE)   /* Left one empty line for new enviroment */
       {
       fputc('%', table);
       fputc('%', strip);
       NewLine(this);
       }
-  if(char_on_line==true)
+  if(char_on_line==CHAR_PRESENT)
       {
       NewLine(this);
       }
@@ -136,7 +136,7 @@ void TconvertedPass1_602::HeaderFooter602(const string &args, char He_Fo)
   flag = HeaderText;
   envir = ' ';
   ActualPos = ftell(wpd);
-  char_on_line = -10;
+  char_on_line = FIRST_CHAR_MINIPAGE;
   while(*Header_Text!=0)
 	{
 	by=*Header_Text++;
@@ -146,7 +146,7 @@ void TconvertedPass1_602::HeaderFooter602(const string &args, char He_Fo)
 	}
 
   Close_All_Attr(attr,strip);
-  if(char_on_line==true)
+  if(char_on_line==CHAR_PRESENT)
      {
      line_term = 's';    	//Soft return
      NewLine(this);
@@ -160,7 +160,7 @@ void TconvertedPass1_602::HeaderFooter602(const string &args, char He_Fo)
   attr = OldAttr;		// Restore backuped attributes
   flag = OldFlag;
   envir = OldEnvir;
-  char_on_line = -10;		// stronger false;
+  char_on_line = FIRST_CHAR_MINIPAGE;	// stronger false;
   recursion--;
 
   strcpy(ObjType,(He_Fo & 3)<=1?"Header":"Footer");
