@@ -17,15 +17,9 @@
 
 #define LineLength   80		/* Split lines after more than LineLength charcters */
 
-#define VersionWP2L	"3.104"
-#define VersionDate	"12 Feb 2022"  /* day (space) month (space) full year */
+#define VersionWP2L	"3.107"
+#define VersionDate	"24 May 2022"  /* day (space) month (space) full year */
 
-#ifndef false
- #define  false          0
-#endif
-#ifndef true
- #define  true           1
-#endif
 
 /* Constants for a flag InputPS */
 typedef enum
@@ -249,6 +243,12 @@ public:
   virtual int Convert_first_pass(void)=0;
   virtual int Dispatch(int Func, const void *arg=NULL) {return(-1);}
   void InitMe(FILE *FileIn, FILE *Table, FILE *StripOut, FILE *LogFile, FILE *ErrorFile);
+
+#ifdef __Limited_Proc_Size
+  void PleaseReport(const char *Feature, const char *Type) const {}
+#else
+  void PleaseReport(const char *Feature, const char *Type);
+#endif
 };
 
 void ReportNoMemory(TconvertedPass1 *cq=NULL);
@@ -572,5 +572,23 @@ DWORD utf8_fgetc(FILE *F);
 }
 #endif
 
+typedef enum
+{
+  LEAVE_ONE_EMPTY_LINE = -20,	/**< Left one empty line for new enviroment. */
+  FIRST_CHAR_MINIPAGE =	 -10,	/**< Line before nested enviroment change, first line of minipages. */
+  JUNK_CHARS =		 -1,	/**< The line contains not text directives. */
+  NO_CHAR =	 	  0,	/**< No valid character on the line. */
+  CHAR_PRESENT =	  1,	/**< Valid character placed on the line. */
+} TCharOnLine;
+
+
+typedef enum
+{
+  STYLE_NOTUSED = 0,
+  STYLE_USED = 1
+} TStyle;
+
+
+
 
 #endif
